

% this module is an effort to enhance Erlang to support semaphores. Semaphores plays a essential role in concurrent programming and because Erlang uses asynchronous message passing and
% therefore no native support for semaphores, so this module can be helpful for concurrent programs whicn mainly intend to use shared data.

% By Danesh Daroui
% Danesh.D@bredband.net
% 14 March 2007
% Gothenburg-Sweden



-module(semaphore).
-export([new/1, p/1, v/1, destroy/1]).
-author('Danesh Daroui').
-copyright('Free under GNU license').



% this routine will acts as a server for semaphore to handle all acquiring (p(s)) and releasing (v(s)) operations regarding to SemValue which is semaphore's initialized value
semaphoreServer(SemValue, WaitList)->
	receive
		{'P', PID}->									% an p(s) operation is received
				case (SemValue>0) of						% can semaphore be acquired?
					true->	PID!{'Granted'},				% yes, so inform the request sender
						semaphoreServer(SemValue-1, WaitList);		% run server again with decremented SemValue for other incomming requests
					false->	NewWaitList=lists:append([PID], WaitList),	% no, semaphore can not be acquired, so add PID of sender to WaitingList
						semaphoreServer(SemValue, NewWaitList)		% run server again with new WaitingList
				end;
		{'V'}->										% an v(s) operation is received
				case (length(WaitList)>0) of					% is there any p(s) waiting for semaphore to acquire it?
					true->	hd(WaitList)!{'Granted'},			% send message to first waiting sender about that semaphore is acquired
						semaphoreServer(SemValue, tl(WaitList));	% run server again with new WaitingList
					false->	semaphoreServer(SemValue+1, WaitList)		% there is no p(s) waiting, so just run server again with incremented SemValue
				end;
		{'Destroy'}->	true								% server should be ended
	end.



% this routine will create a concurrent thread of semaphoreServer to serve incomming p() and v(s) for semaphores
new(SemValue)->spawn(fun()->semaphoreServer(SemValue, []) end).					% create server with initilized SemValue and empty WaitingList queue



% this routine will send request to server to acquire semaphore
p(SemServer)->
	SemServer!{'P', self()},								% send the message to server and also address of sender itself to acquire semaphore
	receive											% the process will be blocked until semaphore is acquired
		{'Granted'}-> true								% semaphore is acquired, so process will be unblocked
	end.



% this routine will inform the server that an acquired semaphore is released
v(SemServer)->
	SemServer!{'V'}.									% send the message to server to inform server to release an acquired semaphore



% destroy the semaphore server and close this semaphore
destroy(SemServer)->
	SemServer!{'Destroy'}.



