
% Test module for semaphore module
% this module will test semaphore module by importing it. It will also demonstrate a practical use of semaphores which is "Barrier Synchronization" which is used here to synchronize two
% processes with different speed and destroy them at the same time. This has been done by using a semaphore (initialized by 2). Synchronizer program will try to acquire these semaphores
% and processes will release them. Thus, synchronizer will be blocked until both processes have been arrived to same point which means that they are done.

% By Danesh Daroui
% Danesh.D@bredband.net
% 14 March 2007
% Gothenburg-Sweden



-module(semaphore_test).
-import(semaphore, [new/1, p/1, v/1, destroy/1]).
-export([run/0]).
-author('Danesh Daroui').
-copyright('Free under GNU license').



%wait for a while
sleep(T) ->
        receive
        after
                T->true
        end.



% first process will just acquire and release a semaphore
sem_Test1(Sem, BSem)->
	sleep(300),
	io:format("Process #1 is acquiring the semaphore...~n"),
	p(Sem),
	io:format("Process #1 has acquired the semaphore...~n"),
	sleep(400),
	v(Sem),
	io:format("Process #1 has released the semaphore...~n"),
	v(BSem).



% second process will just acquire and release a semaphore
sem_Test2(Sem, BSem)->
	sleep(200),
	io:format("Process #2 is acquiring the semaphore...~n"),
	p(Sem),
	io:format("Process #2 has acquired the semaphore...~n"),
	sleep(600),
	v(Sem),
	io:format("Process #2 has released the semaphore...~n"),
	v(BSem).



% synchronizer which will destroy semaphores exactly when both processes are done
barrier_Synch(BSem, Sem)->
	p(BSem),
	p(BSem),
	destroy(Sem),
	destroy(BSem).




%run the test
run()->
	Sem=new(1),
	BSem=new(0),
	spawn_link(fun()->sem_Test1(Sem, BSem) end),
	spawn_link(fun()->sem_Test2(Sem, BSem) end),
	spawn_link(fun()->barrier_Synch(BSem, Sem) end).



